//
//  GLTrackManager.h
//  GLGPSKit
//
//  Created by Jeffrey Early on 7/13/08.
//  Copyright 2008 Early Innovations, LLC. All rights reserved.
//

#import <Cocoa/Cocoa.h>
//#import <GLGPSKit/GLGPS.h>
#import <GLGPSKit/GLGeoPoint.h>
#import <GLGPSKit/GLImportSource.h>
//#import <GLGPSKit/GLTrackImporter.h>

extern NSString *GLTrackManagerDidAddNewImportSource;
extern NSString *GLTrackManagerDidFinishBatchImport;
extern NSString *GLTrackManagerDidDeleteObjects;

@class GLGPSBabelImporterPreflight;
@interface GLTrackManager : NSObject {
	NSPersistentStoreCoordinator *persistentStoreCoordinator;
	NSArray *validFileTypes;
	NSPredicate *validFileTypePredicate;
}

// The "Central Store" is the object graph spread out across several stores for optimization reasons.
// This is stored in the Application Support Folder.

/************************************************/
/*		Intialization							*/
/************************************************/

#pragma mark -
#pragma mark Intialization
#pragma mark

// Returns a singleton instance of the trackManager with access to the central SQL store.
// This instance (its managed object context and persistent store coordinator) should be used exclusively
// on the main thread.
+ (GLTrackManager *) sharedTrackManager;

// For exclusive use on background thread. This singleton uses a separate persistent store coordinator
// from the sharedTrackManager.
+ (GLTrackManager *) sharedBackgroundThreadTrackManager;

// Assigns a point to the appropriate store based on the points date. This the heart of the speed optimization
// for the track manager. This function is called by the point itself whenever the date changes. If the point
// was already assigned to a store and saved there, then it is ignored.
+ (void) assignToStore: (GLGeoPoint *) point;

+ (NSString *) cacheFolder;

// By default this is set to the centralPersistentStoreCoordinator, but you can override this as
// long as you do this before requesting (and therefore initializing) the managedObjectContext.
@property(nonatomic, strong) NSPersistentStoreCoordinator *persistentStoreCoordinator;

// Automatically generated on demand when needed, but can be overridden if necessary.
@property(nonatomic, strong) NSManagedObjectContext *managedObjectContext;

// Creates a new context with NSPrivateQueueConcurrencyType and no parent.
- (NSManagedObjectContext *) newManagedObjectContext;

// Creates a new (no!, not new) context with NSMainQueueConcurrencyType and -managedObjectContext as its parent.
//- (NSManagedObjectContext *) newMainQueueContext;
@property(nonatomic, strong) NSManagedObjectContext *mainQueueContext;

// Creates a new context with NSPrivateQueueConcurrencyType and -managedObjectContext as its parent.
//- (NSManagedObjectContext *) newPrivateQueueContext;
@property(nonatomic, strong) NSManagedObjectContext *newPrivateQueueContext;

// save the managedObjectContext 
- (void) save;

/********************************************************/
/*		Deleting Groups, Events, Tracks & Waypoints     */
/********************************************************/

#pragma mark -
#pragma mark Deleting Groups, Events, Tracks & Waypoints
#pragma mark

// This method isn't very thorough, but it does get the worst of it, eliminating the majority
// of the faults necessary to delete the entire graph in reasonable time.
- (NSArray *) batchFaultGraphFromImportSource: (GLImportSource *) importSource;

// Semi-efficiently deletes an array of mixed managed object types.
- (void) deleteObjects: (NSArray *) managedObjects;

// A test.
- (void) deletePointsWithIDs: (NSArray *) managedObjectIDs;
- (void) deletePointsWithURIs: (NSArray *) managedObjectURIs;

/********************************************************/
/*		Retrieving Groups, Events, Tracks & Waypoints	*/
/********************************************************/

#pragma mark -
#pragma mark Retrieving Groups, Events, Tracks & Waypoints
#pragma mark

// Objects are fetched and returned on the same thread. Specifically, they are fetched with
// NSConfinementConcurrencyType. This means that these objects can only be used on this thread.

- (NSArray *) tracksForPredicate: (NSPredicate *) predicate;
- (NSArray *) waypointsForPredicate: (NSPredicate *) predicate;

// Returns the most recent imported tracks and waypoints.
@property(strong, readonly) NSArray *recentImports;

/************************************************/
/*		Importing New Tracks & Waypoints		*/
/************************************************/

#pragma mark -
#pragma mark Importing New Tracks & Waypoints
#pragma mark

// A list of the valid file types as strings.
@property(strong, readonly) NSArray *validFileTypes;

// A predicate created using the validFileTypes that allows for quick filtering.
@property(strong, readonly) NSPredicate *validFileTypePredicate;

// This method is actually a quick and dirty look at file types and returns YES if one or more files are valid file types.
- (BOOL) validateFilesAtPaths: (NSArray *) paths;

// Any gpx or tcx files to be imported 
- (void) importFilesAtPaths: (NSArray *) paths;

// Cancels all operations in the operation queue.
- (void) cancelImport;

// Import progress
@property(nonatomic, strong) NSOperationQueue *trackImportQueue;
@property BOOL trackImportIsInProgress;
@property BOOL trackImportIsIndeterminateProgress;
@property NSUInteger trackImportProgressMin;
@property NSUInteger trackImportProgressMax;
@property NSUInteger trackImportProgressValue;
@property NSUInteger trackImportLastFinishedProgressValue;
@property(nonatomic, copy) NSString *trackImportProgressInfo;
@property(nonatomic, strong) NSTimer *periodicUpdateTimer;

// Sets up an import dialog on the window. If appropriate, it then addes the tasks to the operation queue.
- (void) importWithGPSBabelForWindow: (NSWindow *) window;


// A track importer object ready to import into this manager's context, and this manager as delegate.
// The root object imported is one (or more) events and can be accessed
// with the function -(NSArray *) lastImportedEvents. The context is NOT saved.
//- (GLTrackImporter *) trackImporter;

@property(nonatomic, strong) NSPredicate *recentImportsPredicate;

// Returns a dictionary with two keys, @"waypointPredicate" and @"trackPredicate"
- (NSDictionary *) nearbyPredicatesForDates: (NSArray *) dates;

/************************************************/
/*		GLTrack Points							*/
/************************************************/

#pragma mark -
#pragma mark GLTrack Points
#pragma mark
//
//// returns the nearest point *strictly* preceding in time.
//- (GLGeoPoint *) pointPrecedingTime: (NSDate *) time;
//
//// returns the nearest point following *or equal to* in time.
//- (GLGeoPoint *) pointFollowingTime: (NSDate *) time;

/************************************************/
/*		Bonus Table								*/
/************************************************/

#pragma mark -
#pragma mark Bonus Table
#pragma mark

// Used to store license information and other persistent data that we don't want in the app preferences.
- (id) fetchBonusTableValueForKey: (NSString *) key;
- (void) setBonusTableValue: (id) value forKey: (NSString *) key;

@end
