//
//  GLTrackPickerWindowController.h
//  GLGPSKit
//
//  Created by Jeffrey Early on 2/4/10.
//  Copyright 2010 Early Innovations, LLC. All rights reserved.
//

#import <Cocoa/Cocoa.h>

// This is the controller mediating the GLTrackSourceList and its window.
// The controller is actually composed of several view controllers, each of which
// could be broken up into separate files, etc.

@class GLTrackSourceList, GLTrackManager;
@interface GLTrackPickerWindowController : NSWindowController <NSSplitViewDelegate, NSToolbarDelegate> {
	GLTrackSourceList *trackSourceList;
	id mapController;
	IBOutlet NSSplitView *splitView;
	
	/************************************************/
	/*		Source Tree								*/
	/************************************************/
	IBOutlet NSTreeController *sourceTreeController;	
	IBOutlet NSOutlineView *sourceOutlineView;	
	
	/************************************************/
	/*		Tracks & Waypoints						*/
	/************************************************/
	IBOutlet NSArrayController *tracksAndWaypointsArrayController;
	IBOutlet NSTableView *tracksAndWaypointsTableView;
	
	/************************************************/
	/*		Track & Waypoint Detail View			*/
	/************************************************/
	NSViewController *trackDetailViewController;
	NSViewController *waypointDetailViewController;
	NSViewController *tracksAndWaypointsDetailViewController;
	
	/************************************************/
	/*		Toolbar									*/
	/************************************************/
	NSString *selectedSourceListDescription;
	
	/************************************************/
	/*		Track Import Progress					*/
	/************************************************/
	IBOutlet NSView *importProgressView;
	IBOutlet NSProgressIndicator *progressIndicator;
}

// This the the model object, the track source list.
@property(nonatomic, strong) GLTrackSourceList *trackSourceList;

// The track manager from which the the above is derived.
@property(strong, readonly) GLTrackManager *trackManager;

// Optional mapController.
@property(nonatomic, strong) id mapController;

// The split view separating the source list from the tracks and waypoints list.
// This controller is the delegate.
@property(strong, readonly) NSSplitView *splitView;

/************************************************/
/*		Source Tree								*/
/************************************************/

#pragma mark -
#pragma mark Source Tree
#pragma mark

// This tree gets bound to the sources property of the trackSourceList.
@property(strong, readonly) NSTreeController *sourceTreeController;

// The source list outline view.
@property(strong, readonly) NSOutlineView *sourceOutlineView;

// Zoom-to-fit on the *clicked* row (not necessarily the selected row). Used by the contextual menu.
- (IBAction) sourceViewZoomToFitAction: (id) sender;

// Reveal the file source of the *clicked* row (not necessarily the selected row). Used by the contextual menu.
- (IBAction) sourceViewRevealAction: (id) sender;

// Remove the contents of the *clicked* row (not necessarily the selected row). Used by the contextual menu.
- (IBAction) sourceViewRemoveAction: (id) sender;

// Remove the contents of the *selected* row. Used by the outline view.
- (IBAction) removeSelectedSourceListAction: (id) sender;

// Confirms the intention to delete with the user, then deletes the source list.
- (void) alertToDeleteSourceList: (id) sourceList;

// This runs through the outline view nodes and makes sure that their expanded property is in line with model.
// Outline view resets to a closed state under certain conditions.
- (void) expandItems;

/************************************************/
/*		Tracks & Waypoints						*/
/************************************************/

#pragma mark -
#pragma mark Tracks & Waypoints
#pragma mark

// This array controller gets bound to the tracks and waypoints property of the trackSourceList.
@property(strong, readonly) NSArrayController *tracksAndWaypointsArrayController;

// The corresponding track and waypoint table view
@property(strong, readonly) NSTableView *tracksAndWaypointsTableView;

// Zoom-to-fit on the *clicked* row (not necessarily the selected row). Used by the contextual menu.
- (IBAction) tableViewZoomToFitAction: (id) sender;

// Reveal the file source of the *clicked* row (not necessarily the selected row). Used by the contextual menu.
- (IBAction) tableViewRevealAction: (id) sender;

// Remove the contents of the *clicked* row (not necessarily the selected row). Used by the contextual menu.
- (IBAction) tableViewRemoveAction: (id) sender;

// Remove the contents of the *selected* row. Used by the table view.
- (IBAction) removeSelectedTracksAndWaypointsAction: (id) sender;

// Confirms the intention to delete with the user, then deletes the track or waypoint.
- (void) alertToDeleteTrackOrWaypoint: (id) anObject;

/************************************************/
/*		Source & Tracks/Waypoint Shared			*/
/************************************************/

#pragma mark -
#pragma mark Source & Tracks/Waypoint Shared
#pragma mark

// We serve as the delegate for the sourceOutlineView and tracksAndWaypointsTableView's contextual menu. This is called before the menu is shown.
- (void)menuNeedsUpdate:(NSMenu *)menu;

// The alert used by for removing source lists and the one used for removing tracks & waypoints call this function.
- (void)alertDidEnd:(NSAlert *)alert returnCode:(int)returnCode contextInfo:(void *)contextInfo;

/************************************************/
/*		Track & Waypoint Detail View			*/
/************************************************/

#pragma mark -
#pragma mark Track & Waypoint Detail View
#pragma mark

@property(strong, readonly) NSViewController *trackDetailViewController;
@property(strong, readonly) NSViewController *waypointDetailViewController;
@property(nonatomic, strong) NSViewController *tracksAndWaypointsDetailViewController;

// This is called when the trackAndWaypointArray selection changes in order to provide context dependent detail views.
- (void) reloadTracksAndWaypointsDetailView;

/************************************************/
/*		Toolbar									*/
/************************************************/

#pragma mark -
#pragma mark Toolbar
#pragma mark

// Used to display the number of tracks, waypoints and trackpoints being shown
@property(nonatomic, copy) NSString * selectedSourceListDescription;

// Zooms to fit the current selection, be it a single track or a whole source list.
- (IBAction) zoomToFit: (id) sender;

// NSToolbar delegate functions
- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar itemForItemIdentifier:(NSString *)itemIdentifier willBeInsertedIntoToolbar:(BOOL)flag;
- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar*)toolbar;
- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar*)toolbar;

/************************************************/
/*		Track Import					*/
/************************************************/

#pragma mark -
#pragma mark Track Import
#pragma mark

- (IBAction) downloadFromGPSAction: (id) sender;
- (IBAction) importFilesAction: (id) sender;
- (IBAction) cancelImportAction: (id) sender;

@property(strong, readonly) NSView *importProgressView;
- (void) importInProgressDidChange;

@end

@interface ImportProgressView : NSView
@end


/************************************************/
/*		TracksAndWaypointsTableView				*/
/************************************************/

#pragma mark -
#pragma mark TracksAndWaypointsTableView
#pragma mark

// Special subclass to forward delete requests
@interface TracksAndWaypointsTableView : NSTableView
@end
